#!/usr/bin/env python
# -*- coding: utf-8 -*-

# ----------------------------------------------------------------------------
# pyjama - python jamendo audioplayer
# Copyright (c) 2008 Daniel Nögel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ----------------------------------------------------------------------------

######################################################################
#                                                                    #
#                     Some default Layouts                           #
#                                                                    #
######################################################################    
#
# Top Layout - main album browser
#

import gtk
import os
import urllib
from time import strftime, gmtime, time, sleep
from math import floor
import hashlib

from modules import clWidgets, clThreadedDownload, clEntry, functions


#
# Plugin
#
class main():
    def __init__(self, pyjama):
        self.pyjama = pyjama
        self.pyjama.Events.connect_event("alldone", self.alldone)

    def alldone(self):
        self.pyjama.layouts.register_layout("radios", RadioBrowser(self.pyjama))

        #
        # Add menu entry to Extras-menu
        #
        menu = self.pyjama.window.menubar
        entry = menu.append_entry(menu.get_rootmenu("Browse"), _("Radios"), "Radios")
        entry.set_sensitive(False)
        entry.connect("activate", self.ev_show_radio_layout)
        menu.set_item_image(entry, os.path.join(functions.install_dir(), "images", "radio.png"))
#        import_star = menu.append_entry(menu.get_rootmenu("Extras"), _("Import Stared Albums"), "ImportStar")
#        import_star.connect("activate", self.ev_import_stared_albums)
#        menu.set_item_image(import_star, gtk.STOCK_NETWORK) 


    def ev_show_radio_layout(self, widget):
        self.pyjama.layouts.show_layout("radios", 1, 50)

#
# Radio Widget
#
class RadioWidget(gtk.Frame):
    def __init__(self, pyjama, radio_id, name, image):
        self.pyjama = pyjama
        gtk.Frame.__init__(self)

        img = gtk.Image()
        img.show()

        lbl = clWidgets.MyLinkButton(name, name)
        lbl.show()
        self.set_label_widget(lbl)
        self.set_label_align(0.5, 0.5)
        self.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
        self.set_size_request(150,160) #(150,160) before controls

        # Event-Box
        self.eventFrame = gtk.EventBox()
        self.eventFrame.set_above_child(False) # ???
        self.eventFrame.add_events (gtk.gdk.ENTER_NOTIFY_MASK)
        self.eventFrame.add_events (gtk.gdk.LEAVE_NOTIFY_MASK)
#        self.eventFrame.connect("enter-notify-event", self.show_vbControl)
#        self.eventFrame.connect("leave-notify-event", self.hide_vbControl)
#        self.eventFrame.connect("button_release_event", self.image_clicked, radio_id)
        self.eventFrame.show()

        self.eventFrame.add(img)
        self.add(self.eventFrame)

        # Image
        md5hash = hashlib.md5(image).hexdigest()
        imagepath = os.path.join(pyjama.home, "images", md5hash)

        if not os.path.exists(imagepath):
            try:
                urllib.urlretrieve(image, imagepath)
                pb = gtk.gdk.pixbuf_new_from_file_at_size(imagepath, 100, 100)
#                img.set_from_file(imagepath)
                img.set_from_pixbuf(pb)
            except IOError:
                print ("Could not get album cover from jamendo")
                try:
                    img.set_from_file(os.path.join(functions.install_dir(), "images", "radio.png"))
                except:
                    img.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_DIALOG)
        else:
            try:
                pb = gtk.gdk.pixbuf_new_from_file_at_size(imagepath, 100, 100)
#                img.set_from_file(imagepath)
                img.set_from_pixbuf(pb)
            except:
                try:
                    img.set_from_file(os.path.join(functions.install_dir(), "images", "radio.png"))
                except:
                    img.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_DIALOG)


        


#
# Layout
#
class RadioBrowser(gtk.Layout):

    def __init__(self, pyjama):
        self.pyjama = pyjama
        
        gtk.Layout.__init__(self)
        self.set_size(700,400)                
        
        # Some default values        
        self.results_per_page = 10
        self.ordering = "ratingweek"
        self.tag = "all"
        self.cur_page = 1     

        
        self.radioinfos={}    
        
        self.pyjama.Events.connect_event("scrolled_window_resized", self.ev_scrolled_window_resized)

        # might be obsolet
        self.pyjama.window.setcolor(self)

    def draw(self, page, num, *args):
        # Checking if next / prev page are available
#        self.check_next_possible(page, num)
#        self.check_prev_possible(page)

        query = "id+name+idstr+image/radio/json/?n=%s&p=%s" % (num, page)
        ret  = self.pyjama.jamendo.query(query)
        if ret is None:
            pass
        if ret < 0:
            pass
        if ret:
            counter = 0
            for dic in ret:
                tmp = RadioWidget(self.pyjama, dic['id'], dic['idstr'], dic['image'])
                tmp.show()
                tmp.eventFrame.connect("button_release_event", self.ev_play_radio, dic['id'])
                self.put(tmp, counter, counter)
                self.radioinfos[counter] = tmp
                counter += 1
    

        self.arrange_topoftheweek()
        self.show()

    def ev_play_radio(self, widget, event, radio_id):
        query = "track_id/track/jsonpretty/radio_track_inradioplaylist/?order=numradio_asc&radio_id=%i" % radio_id
        ret = self.pyjama.jamendo.query(query)
        if ret is None:
            print ("2 fast")
            return
        if ret < 0:
            print ("Error")
            return
        if ret == []:
            print ("No tracks in this radio")
            return

        tracks = self.pyjama.db.get_multiple_trackinfos(ret)
        for track_id in ret:
            for track in tracks:
                if track.id == track_id:
                    self.pyjama.add2playlist(track)

        
    def arrange_topoftheweek(self):
        if self.radioinfos == {}: return None

        width = self.pyjama.window.scrolledwindow_width
        hspace = 20
        vspacer = 20
        
        imgwidth = 150
        imgheight = 160
        
        starty = -10
        
        y = 0
        x = 0
        for counter in self.radioinfos:
            if ((x+1) * imgwidth) + hspace*(x+1) >= width:
                y += 1
                x = 0
            self.move(self.radioinfos[counter], (imgwidth * x) + hspace*(x+1), (vspacer*(y+1) + (y * imgheight))+starty)
            x += 1
        height = (y+1) * (imgheight + vspacer) +starty
        self.set_size(width-20,height+vspacer)

    # Possible Modi:
    #
    # rating, ratingweek, ratingmonth
    # date
    # downloaded, listened, starred, playlisted
    #

    def check_next_possible(self, page=None, rpp=None):
        if page == None: page = self.cur_page
        if rpp == None: rpp = self.results_per_page
        if page >= floor(self.pyjama.db.albums / rpp) - 2:
            self.pyjama.layouts.toolbars['top'].sbNextPage.set_sensitive(False)
        else:
            self.pyjama.layouts.toolbars['top'].sbNextPage.set_sensitive(True)

    def check_prev_possible(self, page=None):
        if page == None: page = self.cur_page
        if page == 1:
            self.pyjama.layouts.toolbars['top'].sbPrevPage.set_sensitive(False)
        else:
            self.pyjama.layouts.toolbars['top'].sbPrevPage.set_sensitive(True)
        
    def ev_scrolled_window_resized(self):
        #
        # Rearrange layout
        #
        self.arrange_topoftheweek()
    #
    # Actually this toolbar is just a hbox...
    #
    class ToolBar(gtk.HBox):
        def __init__(self, pyjama):
            gtk.HBox.__init__(self)
            self.pyjama = pyjama
            self.layout = self.pyjama.layouts.layouts['radios']
            
            
